# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"make_differentiation_matrices": "00_core.ipynb",
         "est_wrapped_gradient": "00_core.ipynb",
         "p_shrink": "00_core.ipynb",
         "make_laplace_kernel": "00_core.ipynb",
         "unwrap": "00_core.ipynb",
         "load_dem_rsc": "01_loading.ipynb",
         "load_interferogram": "01_loading.ipynb",
         "load_gdal": "01_loading.ipynb",
         "RSC_KEY_TYPES": "01_loading.ipynb",
         "get_cli_args": "02_cli.ipynb",
         "main": "02_cli.ipynb"}

modules = ["core.py",
           "loading.py",
           "cli.py"]

doc_url = "https://scottstanie.github.io/spurs/"

git_url = "https://github.com/scottstanie/spurs/tree/master/"

def custom_doc_links(name):
    return None
